"""Tests for login redirects"""

import requests
from tornado.httputil import url_concat

from notebook.tests.launchnotebook import NotebookTestBase


class LoginTest(NotebookTestBase):
    def login(self, next):
        first = requests.get(self.base_url() + "login")
        first.raise_for_status()
        resp = requests.post(
            url_concat(
                self.base_url() + "login",
                {'next': next},
            ),
            allow_redirects=False,
            data={
                "password": self.token,
                "_xsrf": first.cookies.get("_xsrf", ""),
            },
            cookies=first.cookies,
        )
        resp.raise_for_status()
        return resp.headers['Location']

    def test_next_bad(self):
        for bad_next in (
            "//some-host",
            "//host" + self.url_prefix + "tree",
            "https://google.com",
            "/absolute/not/base_url",
            "///jupyter.org",
            "/\\some-host",
        ):
            url = self.login(next=bad_next)
            self.assertEqual(url, self.url_prefix)
        assert url

    def test_next_ok(self):
        for next_path in (
            "tree/",
            self.base_url() + "has/host",
            "notebooks/notebook.ipynb",
            "tree//something",
        ):
            if "://" in next_path:
                expected = next_path
            else:
                expected = self.url_prefix + next_path

            actual = self.login(next=expected)
            self.assertEqual(actual, expected)



#-----------------------------------------------------------------------------
#  Copyright (c) Jupyter Development Team
#
#  Distributed under the terms of the BSD License.  The full license is in
#  the file LICENSE, distributed as part of this software.
#-----------------------------------------------------------------------------

import json
from tornado.log import access_log
from .prometheus.log_functions import prometheus_log_method


def log_request(handler, log=access_log, log_json=False):
    """log a bit more information about each request than tornado's default
    
    - move static file get success to debug-level (reduces noise)
    - get proxied IP instead of proxy IP
    - log referer for redirect and failed requests
    - log user-agent for failed requests
    """
    status = handler.get_status()
    request = handler.request
    if status < 300 or status == 304:
        # Successes (or 304 FOUND) are debug-level
        log_method = log.debug
    elif status < 400:
        log_method = log.info
    elif status < 500:
        log_method = log.warning
    else:
        log_method = log.error

    request_time = 1000.0 * request.request_time()
    ns = dict(
        status=status,
        method=request.method,
        ip=request.remote_ip,
        uri=request.uri,
        request_time=float(f'{request_time:.2f}'),
    )
    msg = "{status} {method} {uri} ({ip}) {request_time:f}ms"
    if status >= 400:
        # log bad referers
        ns['referer'] = request.headers.get('Referer', 'None')
        msg += ' referer={referer}'
    if status >= 500 and status != 502:
        # Log a subset of the headers if it caused an error.
        headers = {}
        for header in ['Host', 'Accept', 'Referer', 'User-Agent']:
            if header in request.headers:
                headers[header] = request.headers[header]
        if log_json:
            log_method("", extra=dict(props=headers))
        else:
            log_method(json.dumps(dict(headers), indent=2))
    if log_json:
        log_method("", extra=dict(props=ns))
    else:
        log_method(msg.format(**ns))
    prometheus_log_method(handler)



"""HTTP handler to shut down the notebook server.
"""
from tornado import web, ioloop
from notebook.base.handlers import IPythonHandler

class ShutdownHandler(IPythonHandler):
    @web.authenticated
    def post(self):
        self.log.info("Shutting down on /api/shutdown request.")
        ioloop.IOLoop.current().stop()


default_handlers = [
    (r"/api/shutdown", ShutdownHandler),
]



from unittest import TestCase
from ipython_genutils.tempdir import TemporaryDirectory
from ..largefilemanager import LargeFileManager
import os
from tornado import web


def _make_dir(contents_manager, api_path):
    """
    Make a directory.
    """
    os_path = contents_manager._get_os_path(api_path)
    try:
        os.makedirs(os_path)
    except OSError:
        print(f"Directory already exists: {os_path!r}")


class TestLargeFileManager(TestCase):

    def setUp(self):
        self._temp_dir = TemporaryDirectory()
        self.td = self._temp_dir.name
        self.contents_manager = LargeFileManager(root_dir=self.td)

    def make_dir(self, api_path):
        """make a subdirectory at api_path

        override in subclasses if contents are not on the filesystem.
        """
        _make_dir(self.contents_manager, api_path)

    def test_save(self):

        cm = self.contents_manager
        # Create a notebook
        model = cm.new_untitled(type='notebook')
        name = model['name']
        path = model['path']

        # Get the model with 'content'
        full_model = cm.get(path)
        # Save the notebook
        model = cm.save(full_model, path)
        assert isinstance(model, dict)
        self.assertIn('name', model)
        self.assertIn('path', model)
        self.assertEqual(model['name'], name)
        self.assertEqual(model['path'], path)

        try:
            model = {'name': 'test', 'path': 'test', 'chunk': 1}
            cm.save(model, model['path'])
        except web.HTTPError as e:
            self.assertEqual('HTTP 400: Bad Request (No file type provided)', str(e))

        try:
            model = {'name': 'test', 'path': 'test', 'chunk': 1, 'type': 'notebook'}
            cm.save(model, model['path'])
        except web.HTTPError as e:
            self.assertEqual('HTTP 400: Bad Request (File type "notebook" is not supported for large file transfer)', str(e))

        try:
            model = {'name': 'test', 'path': 'test', 'chunk': 1, 'type': 'file'}
            cm.save(model, model['path'])
        except web.HTTPError as e:
            self.assertEqual('HTTP 400: Bad Request (No file content provided)', str(e))

        try:
            model = {'name': 'test', 'path': 'test', 'chunk': 2, 'type': 'file',
                     'content': 'test', 'format': 'json'}
            cm.save(model, model['path'])
        except web.HTTPError as e:
            self.assertEqual("HTTP 400: Bad Request (Must specify format of file contents as 'text' or 'base64')",
            				 str(e))

        # Save model for different chunks
        model = {'name': 'test', 'path': 'test', 'type': 'file',
                 'content': 'test==', 'format': 'text'}
        name = model['name']
        path = model['path']
        cm.save(model, path)

        for chunk in (1, 2, -1):
            for fm in ('text', 'base64'):
                full_model = cm.get(path)
                full_model['chunk'] = chunk
                full_model['format'] = fm
                model_res = cm.save(full_model, path)
                assert isinstance(model_res, dict)

                self.assertIn('name', model_res)
                self.assertIn('path', model_res)
                self.assertNotIn('chunk', model_res)
                self.assertEqual(model_res['name'], name)
                self.assertEqual(model_res['path'], path)

        # Test in sub-directory
        # Create a directory and notebook in that directory
        sub_dir = '/foo/'
        self.make_dir('foo')
        model = cm.new_untitled(path=sub_dir, type='notebook')
        name = model['name']
        path = model['path']
        model = cm.get(path)

        # Change the name in the model for rename
        model = cm.save(model, path)
        assert isinstance(model, dict)
        self.assertIn('name', model)
        self.assertIn('path', model)
        self.assertEqual(model['name'], 'Untitled.ipynb')
        self.assertEqual(model['path'], 'foo/Untitled.ipynb')
